
/*
 * $Id: acconfig.h,v 1.9.6.4 1999/11/08 05:31:42 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */


#ifndef PG_CONFIG_FILE_H
#define PG_CONFIG_FILE_H

@TOP@

/* host type from configure */
#define CONFIG_HOST_TYPE "-unknown-"


/* negative or zero if no probing has been done or probing failed */
#define PROBED_MAXFD -1

/* negative or zero if we failed to detect it */
#define DEFAULT_FD_SETSIZE -1


/* sighandler prototype (e.g. "void SignalHandler(...)" on IRIX */
#undef SIGNAL_HANDLER_TYPE

/* various types */
#undef HAVE_TYPE_RUSAGE
#undef HAVE_TYPE_RLIM_T

/* Some systems use socklen_t typedef for some socket operations.
 * Socklen_t may conflict with "int" that is also used.
 */
#undef HAVE_TYPE_SOCKLEN_T

/* sockaddr structure has sa_len member */
#undef HAVE_SA_LEN

/* tm structure has tm_gmtoff member */
#undef HAVE_TM_GMTOFF

/* how time zone global variable looks like (timezone, _timezone, etc.) */
#undef HAVE_TIMEZONE

/* must explicitly declare timezone global as extern */
#undef HAVE_EXTERN_TIMEZONE

/* here are some typedefs that configure script might be missing */
#undef rlim_t


@BOTTOM@

/* various post-processing fixes and adjustments */

// increase FD_SETSIZE if needed and OS supports FD_SETSIZE redifinition
#if !defined(__linux__) && !defined(linux)
#	if PROBED_MAXFD > 0 && DEFAULT_FD_SETSIZE > 0 && PROBED_MAXFD > DEFAULT_FD_SETSIZE
#		ifdef FD_SETSIZE
#			warning FD_SETSIZE is already defined
#		else
#			define FD_SETSIZE PROBED_MAXFD
#		endif
#	endif
#endif

// define socklen_t if not defined for us
#ifndef HAVE_TYPE_SOCKLEN_T
typedef int socklen_t;
#endif

// define SignalHandler type
typedef SIGNAL_HANDLER_TYPE;

// define xtimezone to be whatever ./configure told us
#if defined(HAVE_TIMEZONE)
#	define xtimezone HAVE_TIMEZONE
#else
#	define xtimezone 0
#endif

// what file scanner (select(2), poll(2), etc.) use by default
// note: our current heuristic will fail in many environments
#ifdef HAVE_POLL
#	define PG_PREFFERED_FILE_SCANNER Poll
#else
#	define PG_PREFFERED_FILE_SCANNER Select
#endif

#ifndef HAVE_TYPE_RLIM_T
typedef int rlim_t;
#endif

// define CONFIG_HOST_OS_* macros (shortened to CFG_HOS_*)
// this code is similar to Squid's include/config.h.in
#if defined(__sun__) || defined(__sun)                
#	define CFG_HOS_SUN
#	if defined(__SVR4)                                  
#		define CFG_HOS_SOLARIS
#	else                                                
#		define CFG_HOS_SUNOS
#	endif
#elif defined(__sgi__) || defined(sgi) || defined(__sgi) 
#	define CFG_HOS_SGI
#	if !defined(_SVR4_SOURCE)
#		define _SVR4_SOURCE                
#	endif
#elif defined(__hpux)                           
#	define CFG_HOS_HPUX
#	define CFG_HOS_SYSV
#elif defined(__osf__)                          
#	define CFG_HOS_OSF
#elif defined(__ultrix)                                
#	define CFG_HOS_ULTRIX
#elif defined(_AIX)                                  
#	define CFG_HOS_AIX
#elif defined(__linux__)                        
#	define CFG_HOS_LINUX
#elif defined(__FreeBSD__)                        
#	define CFG_HOS_FREEBSD
#elif defined(__NeXT__)
#	define CFG_HOS_NEXT
#elif defined(__bsdi__)
#	define CFG_HOS_BSDI                         
#elif defined(__NetBSD__)
#	define CFG_HOS_NETBSD
#elif defined(__CYGWIN32__) || defined(WIN32) || defined(WINNT) || defined(__WIN32__) || defined(__WIN32)
#	define CFG_HOS_MSWIN
#elif defined(sony_news) && defined(__svr4)
#	define CFG_HOS_NEWSOS6
#elif defined(__EMX__) || defined(OS2) || defined(__OS2__)
#	define CFG_HOS_OS2
#endif

#endif /* PG_CONFIG_FILE_H */
