
/*
 * $Id: PhaseLx.h,v 1.8.6.1 1999/10/11 21:44:43 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_STAT_PHASE_LOG_EXTRACTOR_H
#define PG_STAT_PHASE_LOG_EXTRACTOR_H

#include "IntvlLx.h"
#include "StatPhaseRec.h"

class PhaseLxRec: public LxRec {
	public:
		virtual const StatIntvlRec &intvl() const { return theRec; }
		virtual const StatPhaseRec &phase() const { return theRec; }

	protected:
		virtual const char *type() const { return "phase"; }

	public:
		StatPhaseRec theRec;
};

// extracts one object from an entry
class PhaseLx: public IntvlLx {
	private:
		typedef void (PhaseLx::*Exor)(const StatPhaseRec &p, ostream &os) const;

	public:
		PhaseLx();

		virtual bool configure(const String &aName, const LxParams &params);
		virtual void extract(const LxRec &r, ostream &os);

	protected:
		void exName(const StatPhaseRec &p, ostream &os) const { os << p.name(); }

		void exConnUseH(const StatPhaseRec &p, ostream &os) const;
		void exConnTtlH(const StatPhaseRec &p, ostream &os) const;
		void exConnCloseActUH(const StatPhaseRec &p, ostream &os) const { printHistogram(p.theConnClose.useHist(ConnCloseStat::ckActive), os); }
		void exConnCloseActTH(const StatPhaseRec &p, ostream &os) const { printHistogram(p.theConnClose.ttlHist(ConnCloseStat::ckActive), os); }
		void exConnCloseFrgnUH(const StatPhaseRec &p, ostream &os) const { printHistogram(p.theConnClose.useHist(ConnCloseStat::ckIdleForeign), os); }
		void exConnCloseFrgnTH(const StatPhaseRec &p, ostream &os) const { printHistogram(p.theConnClose.ttlHist(ConnCloseStat::ckIdleForeign), os); }
		void exConnCloseLclUH(const StatPhaseRec &p, ostream &os) const { printHistogram(p.theConnClose.useHist(ConnCloseStat::ckIdleLocal), os); }
		void exConnCloseLclTH(const StatPhaseRec &p, ostream &os) const { printHistogram(p.theConnClose.ttlHist(ConnCloseStat::ckIdleLocal), os); }

		void exSoReadSzH(const StatPhaseRec &p, ostream &os) const { printHistogram(p.theSockRdSzH, os); }
		void exSoWriteSzH(const StatPhaseRec &p, ostream &os) const { printHistogram(p.theSockWrSzH, os); }

		void exRepSzH(const StatPhaseRec &p, ostream &os) const { printHistogram(p.theRepSzH, os); }

		void exAllRptmH(const StatPhaseRec &p, ostream &os) const;
		void exHitRptmH(const StatPhaseRec &p, ostream &os) const { printHistogram(p.theHitRptmH, os); }
		void exMissRptmH(const StatPhaseRec &p, ostream &os) const { printHistogram(p.theMissRptmH, os); }

		void exRptmVsSz(const StatPhaseRec &p, ostream &os) const;
		void exConnTtlVsUse(const StatPhaseRec &p, ostream &os) const;

	protected:
		void printHistogram(const Histogram &h, ostream &os) const;
		void printVsLine(const Histogram &h, int pStep, ostream &os) const;
		void printTimeVsHist(const TimeVsSmthHist &vsh, int min, const String &type, ostream &os) const;

	private:
		Exor theExor; // set at configure time
};

#endif
