
/*
 * $Id: lr.cc,v 1.9.4.1 1999/12/11 23:38:51 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <iomanip.h>

#include "CmdLine.h"
#include "opts.h"

#include "ILog.h"
#include "polyLogTags.h"
#include "gadgets.h"


class MyOpts: public OptGrp {
	public:
		MyOpts():
			theHelpOpt(this,      "help",           "list of options"),
			theOutFileName(this,  "out <file>",     "redirect console output", "-")
			{}

		virtual ostream &printAnonym(ostream &os) const;
		virtual bool parseAnonym(const Array<const char *> &opts);
		virtual bool canParseAnonym() const { return true; }

	public:
		HelpOpt theHelpOpt;
		StrOpt theOutFileName;

		Array<String*> theFileNames;
};

static MyOpts TheOpts;
static ILog *TheInLog = 0;


ostream &MyOpts::printAnonym(ostream &os) const {
	return os << "<log_file_name> ...";
}

bool MyOpts::parseAnonym(const Array<const char *> &opts) {
	for (int i = 0 ; i < opts.count(); ++i)
		theFileNames.append(new String(opts[i]));
	// add default fname if none are specified
	if (!theFileNames.count())
		theFileNames.append(new String("-"));
	return theFileNames.count() > 0;
}


static
ostream &loghere(ostream &os) {
	return os << setw(7) << TheInLog->progress().time() << "| ";
}

static
void doEntry(const LogEntryPx &px) {
	if (px.theTag == lgComment) {
		String s;
		TheInLog->gets(s);
		cout << loghere << s << endl;
	}
}

static
void configureLogs(int prec) {
	if (TheOpts.theOutFileName && strcmp(TheOpts.theOutFileName, "-"))
		cout = cerr = clog = *new ofstream(TheOpts.theOutFileName);

	configureStream(cout, prec);
	configureStream(cerr, prec);
	configureStream(clog, prec);
}

// set some general stuff and
// propogate cmd line options to corresponding objects
static
void configure() {

	configureLogs(2);
}

int main(int argc, char *argv[]) {

	CmdLine cmd;
	cmd.configure(Array<OptGrp*>() << &TheOpts);
	if (!cmd.parse(argc, argv) || !TheOpts.validate() || !TheOpts.validate())
		return -1;

	configure();

	for (int i = 0; i < TheOpts.theFileNames.count(); ++i) {
		const String &fname = *TheOpts.theFileNames[i];
		istream *is = new ifstream(fname);
		TheInLog = new ILog();
		TheInLog->stream(fname, is);

		if (TheOpts.theFileNames.count() > 1)
			cout << endl << "==> " << fname << " <==" << endl << endl;

		while (const LogEntryPx prefix = TheInLog->begEntry()) {
			doEntry(prefix);
			TheInLog->endEntry();
		}

		delete TheInLog;
		delete is;
	}

	return 0;
}
