
/*
 * $Id: GoalSym.cc,v 1.6 1999/09/07 17:47:38 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include "String.h"
#include "BigSize.h"
#include "PglRec.h"
#include "PglIntSym.h"
#include "PglNumSym.h"
#include "PglTimeSym.h"
#include "PglSizeSym.h"
#include "GoalSym.h"


GoalSym::GoalSym(): RecSym("Goal", new PglRec) {

	// add fields

	theRec->bAdd("time", "duration", 0);
	theRec->bAdd("int", "xactions", 0);
	theRec->bAdd("size", "fill_size", 0);
	theRec->bAdd("float", "errors", 0);
}

GoalSym::GoalSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
}

bool GoalSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "Goal";
}

SynSym *GoalSym::clone(const String &type) const {
	if (isA(type))
		return new GoalSym(this->type(), theRec->clone());
	return RecSym::clone(type);
}

Time GoalSym::duration() const {
	return getTime("duration");
}

bool GoalSym::fillSize(BigSize &sz) const {
	SynSymTblItem *szi = 0;
	Assert(theRec);
	Assert(theRec->find("fill_size", szi));
	if (szi->sym())
		sz = ((const SizeSym&)szi->sym()->cast("size")).val();
	return szi->sym();
}

bool GoalSym::xactCount(int &count) const {
	SynSymTblItem *xi = 0;
	Assert(theRec);
	Assert(theRec->find("xactions", xi));
	if (xi->sym())
		count = ((const IntSym&)xi->sym()->cast("int")).val();
	return xi->sym();
}

bool GoalSym::errs(double &ratio, int &count) const {
	SynSymTblItem *ei = 0;
	Assert(theRec);
	Assert(theRec->find("errors", ei));
	if (ei->sym()) {
		double level = ((const NumSym&)ei->sym()->cast("float")).val();
		if (level < 1)
			ratio = level;
		else
			count = (int)level;
	}
	return ei->sym();
}

