
/*
 * $Id: HostsBasedIter.h,v 1.1.2.3 1999/11/08 06:07:09 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_HOSTS_BASED_ITERATOR_H
#define PG_HOSTS_BASED_ITERATOR_H

#include "Array.h"
#include "NetAddr.h"

class HostsBasedSym;

// a configuration of an object often contains 'hosts' array
// the addresses in hosts array should be filtered against "local" ones
// and then coresponding objects should be created based on
// one-for-many configuration and a specific host address
//
// host based iterator hides the complexity of the above process
// the iterator return (object configuration symbol, host) pairs
class HostsBasedIter {
	public:
		typedef Array<HostsBasedSym*> CfgSyms;
		typedef Array<in_addr> Addrs;

	public:
		HostsBasedIter();
		HostsBasedIter(const Addrs *aLocalAddrs);
		virtual ~HostsBasedIter();

		HostsBasedIter &start(const Addrs *localAddrs);

		bool eof() const;
		operator void *() const;
		HostsBasedIter &operator ++();

		const NetAddr &host();   // current host
		int hostIdx();           // sequential host idx within cfgSym

	protected:
		void sync();
		virtual bool belongs(const HostsBasedSym *cfgSym, const NetAddr &host) const;

		virtual HostsBasedSym *cfgSym(int idx) const = 0;
		virtual int cfgCount() const = 0;    // number of cfg symbols

	private:
		NetAddr host(int cidx, int hidx) const;

	protected:
		const Addrs *theLocalAddrs;  // local addresses

		int theCfgIdx;
		int theHostIdx;

	private:
		NetAddr theHost;       // cached value
};

#endif
