
/*
 * $Id: AlarmClock.cc,v 1.6.6.2 1999/11/28 22:49:23 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "xstd.h"

#include "Assert.h"
#include "AlarmClock.h"

AlarmClock TheAlarmClock;

/* Alarm */

void Alarm::ring() {
	Assert(theSleeper);
	theSleeper->wakeUp(*this);
	clear();
}

void Alarm::snooze() {
	Assert(theSleeper);
	theSleeper->snooze(*this);
	clear();
}

void Alarm::set(Time aTime, AlarmUser *aSleeper) {
	Assert(aSleeper);

	theSleeper = aSleeper;
	theTime = aTime;
}


/* AlarmUser */

AlarmUser::AlarmUser(): thePendAlarmCnt(0) {
}

AlarmUser::~AlarmUser() {
	if (thePendAlarmCnt)
		TheAlarmClock.cancelAll(this);
	Assert(!thePendAlarmCnt);
}

void AlarmUser::wakeUp(const Alarm &) {
	thePendAlarmCnt--;
	Assert(thePendAlarmCnt >= 0);
}

// ignore this alarm
void AlarmUser::snooze(const Alarm &) {
	thePendAlarmCnt--;
	Assert(thePendAlarmCnt >= 0);
}

void AlarmUser::sleepTill(Time time) {
	TheAlarmClock.setAlarm(Alarm(time, this));
	thePendAlarmCnt++;
}

void AlarmUser::sleepFor(Time delay) {
	sleepTill(TheClock + delay);
}


/* AlarmClock */

AlarmClock::AlarmClock() {
}

void AlarmClock::setAlarm(const Alarm &alarm) { 
	if (alarm) {
		theQueue.add(alarm);
		if (theRingTime < 0 || alarm.time() < theRingTime)
			noteNewRingTime(alarm.time());
	}
}

void AlarmClock::cancelAll(const AlarmUser *sleeper) {
	if (sleeper->alarmWaiting())
		for (int i = theQueue.count()-1; i >= 0; --i)
			if (theQueue.at(i).sleeper() == sleeper)
				theQueue.at(i).snooze();
}

// returns time left (negative if late)
void AlarmClock::ring() {
	while (!theQueue.empty()) {
		if (Alarm &alarm = theQueue.top()) {
			noteNewRingTime(alarm.time());
			if (theTimeLeft > 0) // not ready yet
				break;
			theRingTime = Time();
			alarm.ring();
		}
		theQueue.skip();
	}
}

void AlarmClock::update(Time curTime) {
	Clock::update(curTime);
	if (theRingTime >= 0) {
		theTimeLeft = theRingTime - theCurTime;
		if (theRingTime <= curTime)
			ring();
	}
}

void AlarmClock::noteNewRingTime(Time newTime) {
	static Time lastRingTime;
	Assert(newTime >= lastRingTime);

	theTimeLeft = newTime - theCurTime;
	theRingTime = newTime;
}
