
/*
 * $Id: BitMask.cc,v 1.3.2.1 1999/11/28 22:50:39 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "xstd.h"

#include <string.h>

#include "BitMask.h"
#include "Assert.h"


Size BitMask::size() const {
	return BinCount(theCapacity) * sizeof(*theBins);
}

void BitMask::resize(int aCapacity) {
	if (aCapacity == theCapacity)
		return;

	delete[] theBins;
	if (aCapacity > 0) {
		theBins = new Bin[BinCount(aCapacity)];
		theCapacity = aCapacity;
		clear();
	} else {
		theBins = 0;
		theCapacity = 0;
	}
}

void BitMask::grow(int aCapacity, int offset) {
	Assert(aCapacity > theCapacity);

	BitMask old;
	old.takeOver(*this);
	this->resize(aCapacity);

	const int y = offset + old.capacity();
	for (int x = offset; x < y; ++x) {
		if (old.isSet(x % old.capacity()))
			this->beSet(x % capacity());
	}
}

void BitMask::clear() {
	if (theBins)
		memset(theBins, 0, size());
}

void BitMask::takeOver(BitMask &from) {
	Assert(!theBins);
	theBins = from.theBins;
	theCapacity = from.theCapacity;

	from.theBins = 0;
	from.theCapacity = 0;
}
