
/*
 * $Id: BigSize.h,v 1.6.6.2 1999/10/31 02:54:44 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef XSTD_BIG_SIZE_H
#define XSTD_BIG_SIZE_H

// BigSize can accommodate values up to 4611686016279904256 bytes
// for 4 byte integers, which is approximately 4 exabytes
// usually used for Size "accumulators"
class BigSize {
	public:
		static BigSize MB(int mb);
		static BigSize MBd(double mb);

	public:
		BigSize(int bytes = -1);

		bool operator ==(const BigSize &sz) const { return theCnt == sz.theCnt && theAcc == sz.theAcc; }
		bool operator !=(const BigSize &sz) const { return theCnt != sz.theCnt || theAcc != sz.theAcc; }
		bool operator  >(const BigSize &sz) const { return theCnt > sz.theCnt || (theCnt == sz.theCnt && theAcc > sz.theAcc); }
		bool operator >=(const BigSize &sz) const { return theCnt > sz.theCnt || (theCnt == sz.theCnt && theAcc >= sz.theAcc); }
		bool operator  <(const BigSize &sz) const { return theCnt < sz.theCnt || (theCnt == sz.theCnt && theAcc < sz.theAcc); }
		bool operator <=(const BigSize &sz) const { return theCnt < sz.theCnt || (theCnt == sz.theCnt && theAcc <= sz.theAcc); }

		// argument (void *) is ignored, '0' is assumed
		bool operator ==(void *) const { return !theCnt && !theAcc; }
		bool operator !=(void *) const { return theCnt || theAcc; }
		bool operator  >(void *) const { return theCnt > 0 || (theCnt == 0 && theAcc  > 0); }
		bool operator >=(void *) const { return theCnt > 0 || (theCnt == 0 && theAcc >= 0); }
		bool operator  <(void *) const { return theCnt < 0 || (theCnt == 0 && theAcc  < 0); }
		bool operator <=(void *) const { return theCnt < 0 || (theCnt == 0 && theAcc <= 0); }

		BigSize &operator +=(const BigSize &sz);
		BigSize &operator +=(int bytes);
		BigSize &operator -=(const BigSize &sz);
		BigSize &operator -=(int bytes);
		BigSize &operator *=(double f);
		double operator /=(const BigSize &sz) const;

		int byte() const; // will panic if overflow
		double byted() const;

		ostream &print(ostream &os) const;

	public: /* log only */
		int theCnt; // counts the accumulated chunks
		int theAcc; // accumulates values in MAX_INT chunks (~2GB)
};


/* operators */

inline
BigSize operator +(BigSize sz1, BigSize sz2) { return sz1 += sz2; }

inline
BigSize operator -(BigSize sz1, BigSize sz2) { return sz1 -= sz2; }

inline
double operator /(BigSize sz1, BigSize sz2) { return sz1 /= sz2; }

inline
BigSize operator *(BigSize sz, double f) { return sz *= f; }

inline
BigSize operator /(BigSize sz, double f) { return sz *= 1/f; }

inline
BigSize operator +(BigSize sz) { return sz; }

inline
BigSize operator -(BigSize sz) { return BigSize::MB(0) - sz; }


inline
ostream &operator <<(ostream &os, const BigSize &sz) { return sz.print(os); }

#endif
