
/*
 * $Id: Poll.h,v 1.4.6.3 1999/11/28 23:13:33 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef XSTD_POLL_H
#define XSTD_POLL_H

#include "fixPoll.h"

#include "Array.h"
#include "FileScanner.h"

// Poll is a File Scanner based on poll(2) system call

// aggregate bit masks together (or not)
#define XPOLL_RD POLLRDNORM
#define XPOLL_WR POLLWRNORM
#define XPOLL_EX (POLLERR | POLLHUP | POLLNVAL)

// pollfd wrapper
// note: optimized for efficient copying and such
class PollFD: public pollfd {
	public:
		PollFD(int aFD = -1);

		void init(int aFD); // must be called to initialize the object

		bool used() const { return events != 0; }
		inline bool interested(IODir dir) const;
		inline bool ready(IODir dir) const;
		
		void set(IODir dir);
		void clear(IODir dir);
};

class Poll: public FileScanner {
	public:
		typedef FileScanUser User;

	public:
		Poll();

		virtual const char *name() const { return "poll"; }

		virtual void configure(int fdLimit);

		virtual FileScanReserv setFD(int fd, IODir dir, User *p);
		virtual void clearFD(int fd, IODir dir);

	protected:
		virtual int sweep(Time *timeout = 0);
		virtual User *readyUser(int fd, IODir dir, int &fd);

	protected:
		Array<PollFD> thePollees;

	private:
		Array<PollFD> theHotFDs; // used during poll call only
};


/* inlined methods */

inline
bool PollFD::interested(IODir dir) const {
	return dir == dirRead ? (events & XPOLL_RD) : (events & XPOLL_WR);
}

inline
bool PollFD::ready(IODir dir) const {
	return dir == dirRead ?
		(revents & (XPOLL_RD | XPOLL_EX)): 
		(revents & (XPOLL_WR | XPOLL_EX));
}

#endif
